@@ -12,7 +12,11 @@ from .mzml_parser import MzmlParser
 
 
 def process_mzml_file(
-    filename: str, out_dir: str, rt_units=None, int_threshold=1000
+    filename: str,
+    out_dir: str,
+    rt_units=None,
+    int_threshold=1000,
+    relative=False
 ):
     """Constructs a parser for the mzML file and extracts information
 
@@ -20,6 +24,8 @@ def process_mzml_file(
         filename {str} -- Name of the mzML file
         out_dir {str} -- Directory to store the output
         rt_units {str} -- Units of mzML file retention times (min or sec)
+        relative {bool} -- Specifies whether ion intensities in final ripper
+            dict are displayed in relative (%) or absolute units.
 
     Keyword Arguments:
         int_threshold {int} -- Intensity threshold for peaks (default: {1000})
@@ -29,5 +35,6 @@ def process_mzml_file(
         filename,
         out_dir,
         rt_units=rt_units,
-        int_threshold=int_threshold
+        int_threshold=int_threshold,
+        relative_intensity=relative
     ).parse_file()
